package top.ninwoo.app.pathcompute;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.boot.Banner;
import org.springframework.boot.WebApplicationType;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.builder.SpringApplicationBuilder;
import org.springframework.stereotype.Service;
import top.ninwoo.app.api.DataPackage;
import top.ninwoo.app.pathcompute.service.FindService;
import top.ninwoo.app.pathcompute.service.FunctionService;
import top.ninwoo.app.pathcompute.service.TransferService;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.Date;

@Service
@SpringBootApplication
public class Pathmain implements ApplicationRunner {

    @Autowired
    private FindService findService;
    @Autowired
    private FunctionService functionService;
    @Autowired
    private TransferService transferService;

    public static void main(String[] args) {
        new SpringApplicationBuilder(Pathmain.class)
                .web(WebApplicationType.NONE) // .REACTIVE, .SERVLET
                .bannerMode(Banner.Mode.OFF)
                .run(args);
    }

    @Override
    public void run(ApplicationArguments args) {
        ServerSocket serverSocket = null;
        InputStream inputStream = null;
        ObjectInputStream objectInputStream = null;
        try {
            serverSocket = new ServerSocket(2020);
            while (true) {
                //接收数据包
                Socket client = serverSocket.accept();
                inputStream = client.getInputStream();
                objectInputStream = new ObjectInputStream(new BufferedInputStream(inputStream));
                Object object = objectInputStream.readObject();
                //接受到数据包
                DataPackage dataPackage = (DataPackage) object;
                System.out.println("receive");
                //构建新的数据包
                DataPackage newDataPackage = dataPackage;
                //得到本机路由表
                String[] route = findService.findRoute(dataPackage.getCurrentSataHop(), dataPackage.getRoute());

                //判断是否为该边的终点,如果是，进行函数计算并创建一个新的数据包,转发
                if (dataPackage.getCurrentSataHop().equals(dataPackage.getDestinSataHop())) {
                    //此时为终点
                    String task = findService.findTask(dataPackage.getCurrentSataHop(), dataPackage.getTaskTable());
                    switch (task) {
                        case "add1":
                            functionService.add1(dataPackage.getData());
                            break;
                        case "add":
                            functionService.add(dataPackage.getData());
                            break;
                        case "addself":
                            functionService.addself(dataPackage.getData());
                            break;
                        case "square":
                            functionService.square(dataPackage.getData());
                            break;
                    }
                    System.out.println("res = "+functionService.getRes());
                    //res不等于-1，说明计算过程正常
                    if (functionService.getRes() != -1) {
                        //写入更新后的数据
                        newDataPackage.setData(functionService.getRes());
                        //找到目的地卫星数组
                        ArrayList<String> destinSataHops = findService.findDestinSataHops(dataPackage.getPathTable(), dataPackage.getCurrentSataHop());
                        if(destinSataHops.get(0).equals("nolast")){
                            System.out.println("运行完毕");
                            break;
                        }
                        for (String destinSataHop : destinSataHops) {
                            //设置目的地址
                            newDataPackage.setDestinSataHop(destinSataHop);
                            //设置下一跳地址
                            newDataPackage.setNextSataHop(findService.findNextHop(destinSataHop, route));
                            //设置本机地址
                            newDataPackage.setCurrentSataHop(dataPackage.getNextSataHop());
                            //创建时间
                            Date date = new Date();
                            long time = date.getTime();
                            //转发
                            transferService.transferPackage(newDataPackage, newDataPackage.getNextSataHop(), time);
                        }
                    }
                } else {//仅作转发的节点
                    newDataPackage.setNextSataHop(findService.findNextHop(dataPackage.getDestinSataHop(), route));
                    newDataPackage.setCurrentSataHop(dataPackage.getNextSataHop());
                    //创建时间
                    Date date = new Date();
                    long time = date.getTime();
                    //转发
                    transferService.transferPackage(newDataPackage, newDataPackage.getNextSataHop(), time);
                }
            }
        } catch (IOException | ClassNotFoundException e) {
            e.printStackTrace(); } finally {
            try {
                if (objectInputStream != null) {
                    objectInputStream.close();
                }
                if (inputStream != null) {
                    inputStream.close();
                }
                if (serverSocket != null) {
                    serverSocket.close();
                }
            } catch (IOException e) {
                e.printStackTrace(); }
        }
    }
}
